/*!
*********************************************************************
* Copyright 2006-2016 VIVOTEK, Inc. All rights reserved.
*
* \file
* AVComposer.h
*
* \brief
* Header file of AVComposer recording module
*
*
* \author
* VIVOTEK
*
*********************************************************************
*/

#ifndef VVTK_SDK_AVCOMPOSER_H
#define VVTK_SDK_AVCOMPOSER_H

#include "common.h"
#include "MainProfile/datapacketdef.h"

#include "AVComposerDefine.h"
#include "AVComposerErrorDef.h"

#ifndef DECLARE_INTERFACE
#if defined(__cplusplus) && !defined(CINTERFACE)

#ifndef DECLSPEC_NOTHROW
#if (_MSC_VER >= 1200)
#define DECLSPEC_NOTHROW __declspec(nothrow)
#else
#define DECLSPEC_NOTHROW
#endif
#endif

#ifndef DECLSPEC_NOVTABLE
#if (_MSC_VER >= 1100)
#define DECLSPEC_NOVTABLE __declspec(novtable)
#else
#define DECLSPEC_NOVTABLE
#endif
#endif

#define STDMETHOD_(type,method)  virtual DECLSPEC_NOTHROW type DEFEXPAPI method
#define PURE                    = 0
#define THIS_
#define THIS                    void
#define DECLARE_INTERFACE(iface)    struct DECLSPEC_NOVTABLE iface

#else

#define STDMETHOD_(type,method) type (DEFEXPAPI *method)
#define PURE
#define THIS_                   INTERFACE* This,
#define THIS                    INTERFACE* This
#define DECLARE_INTERFACE(iface)    typedef struct iface { \
                                    const struct iface##Vtbl* lpVtbl; \
                                } iface; \
                                typedef struct iface##Vtbl iface##Vtbl; \
                                struct iface##Vtbl

#endif
#endif

#undef INTERFACE
#define INTERFACE AVComposer
DECLARE_INTERFACE(AVComposer)
{
	STDMETHOD_(SCODE, Release)(THIS) PURE;

	STDMETHOD_(SCODE, StartRecording)(THIS_ const char* pszFilename, DWORD dwOptions) PURE;
	STDMETHOD_(SCODE, StopRecording)(THIS_ TAVCOMPOSER_CLOSEFILEINFO* ptFileInfo, BOOL bKeepBuffer) PURE;
	STDMETHOD_(SCODE, InputPacket)(THIS_ EAVCOMPOSER_TRACK_TYPE eTrackType, TMediaDataPacketInfoV3* ptMediaPacket, DWORD* pdwFileSize) PURE;

	STDMETHOD_(SCODE, AddProprietaryBox)(THIS_ DWORD dwBoxName, const BYTE* pbyContentBuf, DWORD dwContentLeng) PURE;

	STDMETHOD_(void, SetAuthenticity)(THIS_ const char* pszPassword) PURE;
};

#undef INTERFACE
#define INTERFACE AVComposerW
DECLARE_INTERFACE(AVComposerW)
{
	STDMETHOD_(SCODE, Release)(THIS) PURE;

	STDMETHOD_(SCODE, StartRecording)(THIS_ const wchar_t *pszFilename, DWORD dwOptions) PURE;
	STDMETHOD_(SCODE, StopRecording)(THIS_ TAVCOMPOSER_CLOSEFILEINFO* ptFileInfo, BOOL bKeepBuffer) PURE;
	STDMETHOD_(SCODE, InputPacket)(THIS_ EAVCOMPOSER_TRACK_TYPE eTrackType, TMediaDataPacketInfoV3* ptMediaPacket, DWORD* pdwFileSize) PURE;

	STDMETHOD_(SCODE, AddProprietaryBox)(THIS_ DWORD dwBoxName, const BYTE* pbyContentBuf, DWORD dwContentLeng) PURE;

	STDMETHOD_(void, SetAuthenticity)(THIS_ const char* pszPassword) PURE;
};

#if !defined(__cplusplus) || defined(CINTERFACE)
#define AVComposer_ReleaseHandle(p)                (p)->lpVtbl->Release(p)
#define AVComposer_StartRecording(p, a, b)         (p)->lpVtbl->StartRecording(p, a, b)
#define AVComposer_StopRecording(p, a, b)          (p)->lpVtbl->StopRecording(p, a, b)
#define AVComposer_InputPacket(p, a, b, c)         (p)->lpVtbl->InputPacket(b, a, c)
#define AVComposer_AddProprietaryBox(p, a, b, c)   (p)->lpVtbl->AddProprietaryBox(p, a, b, c)
#define AVComposer_SetAuthenticity(p, a)           (p)->lpVtbl->SetAuthenticity(p, a)
#else
#define AVComposer_ReleaseHandle(p)                (p)->Release()
#define AVComposer_StartRecording(p, a, b)         (p)->StartRecording(a, b)
#define AVComposer_StopRecording(p, a, b)          (p)->StopRecording(a, b)
#define AVComposer_InputPacket(p, a, b, c)         (p)->InputPacket(b, a, c)
#define AVComposer_AddProprietaryBox(p, a, b, c)   (p)->AddProprietaryBox(a, b, c)
#define AVComposer_SetAuthenticity(p, a)           (p)->SetAuthenticity(a)
#endif

#ifdef __cplusplus
extern "C" {
#endif

SCODE DLLAPI AVComposer_CreateHandle(AVComposer** ppAVComposer, DWORD dwMaxFileSize = -1, DWORD dwMaxTimePeriod = -1, AVCOMPOSER_PF_REQUEST_FILENAME pfRequestFilename = NULL, HANDLE hContext = NULL);

SCODE DLLAPI AVComposer_CreateHandleW(AVComposerW** ppAVComposer, DWORD dwMaxFileSize = -1, DWORD dwMaxTimePeriod = -1, AVCOMPOSER_PF_REQUEST_FILENAMEW pfRequestFilenameW = NULL, HANDLE hContext = NULL);

#ifdef __cplusplus
}
#endif

#endif // VVTK_SDK_AVCOMPOSER_H
