#ifndef VVTK_SDK_AVREADER_H
#define VVTK_SDK_AVREADER_H

#include "common.h"
#include "datapacketdef.h"

#define S_AVREADER_EOF				MAKE_SCODE(SEVERITY_SUCCESS, 0, 0x6001)
#define S_AVREADER_NO_PROPER_AUTH	MAKE_SCODE(SEVERITY_SUCCESS, 0, 0x6002)
#define S_AVREADER_AUTH_NOT_MATCH	MAKE_SCODE(SEVERITY_SUCCESS, 0, 0x6003)

typedef enum
{
	AVReaderFaster_1x,
	AVReaderFaster_2x,
	AVReaderFaster_4x,
	AVReaderFaster_8x,
	AVReaderFaster_16x,
	AVReaderFaster_32x,
	AVReaderFaster_64x,
} EAVReaderFaster;

#ifndef DECLARE_INTERFACE
#if defined(__cplusplus) && !defined(CINTERFACE)

#ifndef DECLSPEC_NOTHROW
#if (_MSC_VER >= 1200)
#define DECLSPEC_NOTHROW __declspec(nothrow)
#else
#define DECLSPEC_NOTHROW
#endif
#endif

#ifndef DECLSPEC_NOVTABLE
#if (_MSC_VER >= 1100)
#define DECLSPEC_NOVTABLE __declspec(novtable)
#else
#define DECLSPEC_NOVTABLE
#endif
#endif

#define STDMETHOD_(type,method)  virtual DECLSPEC_NOTHROW type DEFEXPAPI method
#define PURE                    = 0
#define THIS_
#define THIS                    void
#define DECLARE_INTERFACE(iface)    struct DECLSPEC_NOVTABLE iface

#else

#define STDMETHOD_(type,method) type (DEFEXPAPI *method)
#define PURE
#define THIS_                   INTERFACE* This,
#define THIS                    INTERFACE* This
#define DECLARE_INTERFACE(iface)    typedef struct iface { \
                                    const struct iface##Vtbl* lpVtbl; \
                                } iface; \
                                typedef struct iface##Vtbl iface##Vtbl; \
                                struct iface##Vtbl

#endif
#endif

#undef INTERFACE
#define INTERFACE AVReader
DECLARE_INTERFACE(AVReader)
{
	STDMETHOD_(void, Release)(THIS) PURE;

	STDMETHOD_(DWORD, GetMediaType)(THIS) PURE;
	STDMETHOD_(DWORD, GetCreationTime)(THIS) PURE;
	STDMETHOD_(DWORD, GetDuration)(THIS) PURE;
	STDMETHOD_(DWORD, GetTrackNum)(THIS) PURE;
	STDMETHOD_(DWORD, GetFourCC)(THIS_ DWORD dwTrackIdx) PURE;
	STDMETHOD_(DWORD, GetStreamType)(DWORD dwTrackIdx) PURE;
	STDMETHOD_(DWORD, GetTrackID)(THIS_ DWORD dwTrackIdx) PURE;
	STDMETHOD_(DWORD, GetSampleCount)(THIS_ DWORD dwTrackIdx) PURE;
	STDMETHOD_(DWORD, GetTrkDuration)(THIS_ DWORD dwTrackIdx) PURE;
	STDMETHOD_(WORD, GetWidth)(THIS_ DWORD dwTrackIdx) PURE;
	STDMETHOD_(WORD, GetHeight)(THIS_ DWORD dwTrackIdx) PURE;

	STDMETHOD_(SCODE, GetTrackCI)(THIS_ DWORD dwTrackIdx, const BYTE** ppbyCI, DWORD* pdwCISize) PURE;
	STDMETHOD_(SCODE, GetProprietaryBox)(THIS_ DWORD dwBoxName, const BYTE** ppbyBuf, DWORD* pdwSize) PURE;

	STDMETHOD_(SCODE, Read)(THIS_ TMediaDataPacketInfoV3**) PURE;
	STDMETHOD_(SCODE, ReadBackward)(THIS_ TMediaDataPacketInfoV3**) PURE;
	STDMETHOD_(void, RemoveTrack)(THIS_ DWORD dwTrackIdx) PURE;
	STDMETHOD_(SCODE, SetPosition)(THIS_ DWORD dwPosition) PURE;
	STDMETHOD_(SCODE, SetPositionBackward)(THIS_ DWORD dwPosition) PURE;
	STDMETHOD_(void, Seek)(THIS_ DWORD dwTimeInSecond, DWORD dwMilliSecond) PURE;
	STDMETHOD_(void, SeekBackward)(THIS_ DWORD dwTimeInSecond, DWORD dwMilliSecond) PURE;

	STDMETHOD_(void, SetFaster)(THIS_ EAVReaderFaster eFaster) PURE;
	STDMETHOD_(void, SetIFrameOnly)(THIS_ BOOL bIFrameOnly) PURE;
	STDMETHOD_(void, SetAuthenticity)(THIS_ const char* pszPassword) PURE;

	STDMETHOD_(void, DisableTrack)(THIS_ DWORD dwTrackIdx) PURE;
	STDMETHOD_(void, EnableTrack)(THIS_ DWORD dwTrackIdx) PURE;
};

#if !defined(__cplusplus) || defined(CINTERFACE)
#define AVReader_Release(p)                           (p)->lpVtbl->Release(p)
#define AVReader_GetMediaType(p)                      (p)->lpVtbl->GetMediaType(p)
#define AVReader_GetCreationTime(p)                   (p)->lpVtbl->GetCreationTime(p)
#define AVReader_GetDuration(p)                       (p)->lpVtbl->GetDuration(p)
#define AVReader_GetTrackNum(p)                       (p)->lpVtbl->GetTrackNum(p)
#define AVReader_GetFourCC(p, a)                      (p)->lpVtbl->GetFourCC(p, a)
#define AVReader_GetStreamType(p, a)                  (p)->lpVtbl->GetStreamType(p, a)
#define AVReader_GetTrackID(p, a)                     (p)->lpVtbl->GetTrackID(p, a)
#define AVReader_GetSampleCount(p, a)                 (p)->lpVtbl->GetSampleCount(p, a)
#define AVReader_GetTrkDuration(p, a)                 (p)->lpVtbl->GetTrkDuration(p, a)
#define AVReader_GetWidth(p, a)                       (p)->lpVtbl->GetWidth(p, a)
#define AVReader_GetHeight(p, a)                      (p)->lpVtbl->GetHeight(p, a)
#define AVReader_GetTrackCI(p, a, b, c)               (p)->lpVtbl->GetTrackCI(a, b, c)
#define AVReader_GetProprietaryBox(p, a, b, c)        (p)->lpVtbl->GetProprietaryBox(a, b, c)
#define AVReader_OutputPacket(p, a)                   (p)->lpVtbl->Read(p, a)
#define AVReader_OutputPacketBackward(p, a)           (p)->lpVtbl->ReadBackward(p, a)
#define AVReader_RemoveTrack(p, a)                    (p)->lpVtbl->RemoveTrack(p, a)
#define AVReader_SetPosition(p, a)                    (p)->lpVtbl->SetPosition(p, a)
#define AVReader_SetPositionBackward(p, a)            (p)->lpVtbl->SetPositionBackward(p, a)
#define AVReader_SetPositionDirectly(p, a, b)         (p)->lpVtbl->Seek(p, a, b)
#define AVReader_SetPositionDirectlyBackward(p, a, b) (p)->lpVtbl->SeekBackward(p, a, b)
#define AVReader_SetFaster(p, a)                      (p)->lpVtbl->SetFaster(p, a)
#define AVReader_SetIFrameOnly(p, a)                  (p)->lpVtbl->SetIFrameOnly(p, a)
#define AVReader_SetAuthenticity(p, a)                (p)->lpVtbl->SetAuthenticity(p, a)
#define AVReader_DisableTrack(p, a)                   (p)->lpVtbl->DisableTrack(p, a)
#define AVReader_EnableTrack(p, a)                    (p)->lpVtbl->EnableTrack(p, a)
#else
#define AVReader_Release(p)                           (p)->Release()
#define AVReader_GetMediaType(p)                      (p)->GetMediaType()
#define AVReader_GetCreationTime(p)                   (p)->GetCreationTime()
#define AVReader_GetDuration(p)                       (p)->GetDuration()
#define AVReader_GetTrackNum(p)                       (p)->GetTrackNum()
#define AVReader_GetFourCC(p, a)                      (p)->GetFourCC(a)
#define AVReader_GetStreamType(p, a)                  (p)->GetStreamType(a)
#define AVReader_GetTrackID(p, a)                     (p)->GetTrackID(a)
#define AVReader_GetSampleCount(p, a)                 (p)->GetSampleCount(a)
#define AVReader_GetTrkDuration(p, a)                 (p)->GetTrkDuration(a)
#define AVReader_GetWidth(p, a)                       (p)->GetWidth(a)
#define AVReader_GetHeight(p, a)                      (p)->GetHeight(a)
#define AVReader_GetTrackCI(p, a, b, c)               (p)->GetTrackCI(a, b, c)
#define AVReader_GetProprietaryBox(p, a, b, c)        (p)->GetProprietaryBox(a, b, c)
#define AVReader_OutputPacket(p, a)                   (p)->Read(a)
#define AVReader_OutputPacketBackward(p, a)           (p)->ReadBackward(a)
#define AVReader_RemoveTrack(p, a)                    (p)->RemoveTrack(a)
#define AVReader_SetPosition(p, a)                    (p)->SetPosition(a)
#define AVReader_SetPositionBackward(p, a)            (p)->SetPositionBackward(a)
#define AVReader_SetPositionDirectly(p, a, b)         (p)->Seek(a, b)
#define AVReader_SetPositionDirectlyBackward(p, a, b) (p)->SeekBackward(a, b)
#define AVReader_SetFaster(p, a)                      (p)->SetFaster(a)
#define AVReader_SetIFrameOnly(p, a)                  (p)->SetIFrameOnly(a)
#define AVReader_SetAuthenticity(p, a)                (p)->SetAuthenticity(a)
#define AVReader_DisableTrack(p, a)                   (p)->DisableTrack(a)
#define AVReader_EnableTrack(p, a)                    (p)->EnableTrack(a)
#endif

#ifdef __cplusplus
extern "C" {
#endif

SCODE DLLAPI AVReader_Initial(AVReader** ppAVReader, const char* pszFilename, BOOL bDiscardVivo = FALSE);

SCODE DLLAPI AVReader_InitialW(AVReader** ppAVReader, const wchar_t* pszFilename, BOOL bDiscardVivo = FALSE);

#ifdef __cplusplus
}
#endif

#endif // VVTK_SDK_AVREADER_H
