#include "stdafx.h"
#include <memory.h>
#include "AvSyncLiveGetRawWrapper.h"


// Function     : CAvSyncLiveGetRawWrapper::CAvSyncLiveGetRawWrapper
// Description  : Constructor
// Parameter    : 
//              | [in] hDlg                 @Dialog window handle
//              | [in] hDisplay             @Display window handle
//              | [in] hBMP                 @Bitmap handle
//              | [in] pfStatus             @Status callback function
//              | [in] pfDisplay            @Display callback function
//              | [in] dwStatusContext      @Status context
//              | [in] dwDisplayContext     @Display context

// Return value : N/A
//
CAvSyncLiveGetRawWrapper::CAvSyncLiveGetRawWrapper(HWND hDlg, HWND hDisplay, HBITMAP hBMP, LPSTATUSCALLBACK pfStatus, LPDISPLAYCALLBACK pfDisplay, DWORD dwStatusContext, DWORD dwDisplayContext)
{
	m_hAvSync      = NULL;
	m_hLiveChannel = NULL;

	m_dwDecodeFormat = PF_YUV;  // Initialize the decode format as YUV
	
	memset(&m_tParam, 0, sizeof(m_tParam));

	m_tParam.hDlg     = hDlg;
	m_tParam.hDisplay = hDisplay;
	m_tParam.hBMP     = hBMP;

	m_tParam.pfStatus  = pfStatus;
	m_tParam.pfDisplay = pfDisplay;
	
	m_tParam.dwStatusContext  = dwStatusContext;
	m_tParam.dwDisplayContext = dwDisplayContext;

	InitialModule();  // Initialize AvSynchronizer and create live channel
}

// Function     : CAvSyncLiveGetRawWrapper::~CAvSyncLiveGetRawWrapper
// Description  : Destructor
// Parameter    : N/A
// Return value : N/A
//
CAvSyncLiveGetRawWrapper::~CAvSyncLiveGetRawWrapper()
{	
	ReleaseModule();  // Delete live channel and release AvSynchronizer
}

// Function     : CAvSyncLiveGetRawWrapper::InitialModule
// Description  : Initialize AvSynchronizer and create live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveGetRawWrapper::InitialModule()
{	
	SCODE scRet;

	// STEP 1: Initialize AvSynchronizer
	scRet = AvSynchronizer_InitialEx(&m_hAvSync, m_tParam.pfStatus, m_tParam.pfDisplay, m_tParam.hDlg, AUDIOOUT_FOCUS_NORMAL, 0, AVSYNCHRONIZER_VERSION, 0, RGB(0, 0, 0));

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InitialEx fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	// STEP 2: Create live channel
	if	(m_hAvSync != NULL)
	{
		scRet = CreateChannel();

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveGetRawWrapper::ReleaseModule
// Description  : Delete live channel and release AvSynchronizer
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveGetRawWrapper::ReleaseModule()
{
	SCODE scRet;
	
	if	(m_hAvSync != NULL)
	{
		// STEP 1: Delete live channel
		if	(m_hLiveChannel != NULL)
		{
			scRet = DeleteChannel();

			if	(IS_FAIL(scRet))
			{
				return S_FAIL;
			}
		}

		// STEP 2: Release AvSynchronizer
		scRet = AvSynchronizer_Release(&m_hAvSync);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_Release fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveGetRawWrapper::CreateChannel
// Description  : Create live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveGetRawWrapper::CreateChannel()
{	
	// STEP 1: Set channel options
	TCHANNELOPTION tCHOption;

	memset(&tCHOption, 0, sizeof(tCHOption));

	tCHOption.dwFlags = CH_STATUSCB | CH_DISPLAYCB | CH_STATUSCONTEXT | CH_DISPLAYCONTEXT | CH_ALLBORDER | CH_BITMAP;

	tCHOption.pfStatus  = m_tParam.pfStatus; 
	tCHOption.pfDisplay = m_tParam.pfDisplay;

	tCHOption.dwStatusContext = m_tParam.dwStatusContext;
	tCHOption.dwDisplayContext = m_tParam.dwDisplayContext;
	
	tCHOption.dwTopBorderSize    = 20;
	tCHOption.dwLeftBorderSize   = 0;
	tCHOption.dwBottomBorderSize = 0;
	tCHOption.dwRightBorderSize  = 0;

	tCHOption.hBMP     = m_tParam.hBMP;
	tCHOption.hDisplay = m_tParam.hDisplay;
	
	SCODE scRet;

	// STEP 2: Create live channel
	scRet = AvSynchronizer_CreateChannel(m_hAvSync, &m_hLiveChannel, tCHOption);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_CreateChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}
	
	return S_OK;
}

// Function     : CAvSyncLiveGetRawWrapper::DeleteChannel
// Description  : Delete live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveGetRawWrapper::DeleteChannel()
{
	SCODE scRet;

	// Delete live channel 
	scRet = AvSynchronizer_DeleteChannel(m_hAvSync, &m_hLiveChannel);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_DeleteChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncLiveGetRawWrapper::Start
// Description  : Start live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveGetRawWrapper::Start()
{	
	SCODE scRet;

	// Start live channel
	if	(m_hLiveChannel != NULL)
	{
		scRet = AvSynchronizer_StartChannel(m_hLiveChannel, DRAW_CONNECTING);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StartChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveGetRawWrapper::Stop
// Description  : Stop live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveGetRawWrapper::Stop()
{
	SCODE scRet;
	
	// Stop live channel
	if	(m_hLiveChannel != NULL)
	{
		scRet = AvSynchronizer_StopChannel(m_hLiveChannel);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StopChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveGetRawWrapper::UpdateChannelMediaType
// Description  : Update channel media type
// Parameter    :
//              | [in] dwMediaType @Media type
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveGetRawWrapper::UpdateChannelMediaType(DWORD dwMediaType)
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{	
		TUPDATECHANNELOPTION tUpdataCHOption;

		tUpdataCHOption.dwFlags    = UPCH_MEDIATYPE;
		tUpdataCHOption.tMediaType = EMEDIATYPE(dwMediaType);
	
		scRet = AvSynchronizer_UpdateChannelSettings(m_hLiveChannel, tUpdataCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdateChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveGetRawWrapper::UpdateChannelShowBMP
// Description  : Update channel settings to show bitmap
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveGetRawWrapper::UpdateChannelShowBMP()
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{	
		TUPDATECHANNELOPTION tUpdataCHOption;

		tUpdataCHOption.dwFlags = UPCH_SHOW_BMP;
	
		scRet = AvSynchronizer_UpdateChannelSettings(m_hLiveChannel, tUpdataCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdateChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveGetRawWrapper::InputEncodedMediaFrame
// Description  : Input the encoded media frame into channel
// Parameter    :
//              | [in] pMediaDataPacket @Input media data packet info
// Return value :
//              | S_OK                        @Success
//              | S_FAIL                      @FAIL
//              | AVSYNCHRONIZER_E_QUEUE_FULL @Queue full
//
SCODE CAvSyncLiveGetRawWrapper::InputEncodedMediaFrame(TMediaDataPacketInfo *pMediaDataPacket)
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{		
		scRet = AvSynchronizer_InputEncodedMediaFrame(m_hLiveChannel, pMediaDataPacket);

		if (scRet != S_OK)
		{
			// The return value "AVSYNCHRONIZER_E_QUEUE_FULL" is used to notify the caller that the queue is full.
			// When used in DataBrokerAVCallback function, if the callback function recieves "AVSYNCHRONIZER_E_QUEUE_FULL",
			// it returns "DATABROKER_S_FRAME_NOT_HANDLED" back to DataBroker. Then DataBroker stop putting media data into
			// the queue until it recieves "S_OK."
			//
			if	(scRet == AVSYNCHRONIZER_E_QUEUE_FULL)
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer queue full!\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return scRet;
			}
			else
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InputEncodedMediaFrame fail! (ERROR: %X)\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return S_FAIL;
			}
		}
	}

	return S_OK;
}


// Function     : CAvSyncLiveGetRawWrapper::SetDecodeOptions
// Description  : Set decode data format of decoder channel
// Parameter    :
//              | [in] dwFormat             @Decode data format
//              | [in] pfDecodeFrame        @DecodeFrame callback function
//              | [in] dwDecodeFrameContext @DecodeFrame context
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveGetRawWrapper::SetDecodeOptions(DWORD dwFormat, LPDECODEFRAMECALLBACK_EX pfDecodeFrameEx, DWORD dwDecodeFrameContext)
{
	m_dwDecodeFormat = dwFormat;
	
	EPIXELFORMAT ePixelFormat = EPIXELFORMAT(dwFormat);

	SCODE scRet;

	scRet = AvSynchronizer_SetChannelOption(m_hLiveChannel, SETCH_DECODE_DATA_FORMAT, ePixelFormat, 0);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_SetChannelOption fail! (ERROR: %X)", scRet);

		OutputDebugString(m_szErrMsgBuf);

		return S_FAIL;
	}

	// Notice!
	// Here caller should use flag "SETCH_OUTPUT_VIDEO_FRAME" and pass "TRUE" to request the decoded video frame from live channel
	//
	AvSynchronizer_SetChannelOption(m_hLiveChannel, SETCH_OUTPUT_VIDEO_FRAME, (DWORD)TRUE, 0);  // Set "SETCH_OUTPUT_VIDEO_FRAME" to channel
	
	// Notice!
	// Here caller should use flag "SETCH_DECODE_CBEX", provide a context and a callback function with type "LPDECODEFRAMECALLBACK_EX"
	//
	AvSynchronizer_SetChannelOption(m_hLiveChannel, SETCH_DECODE_CBEX, dwDecodeFrameContext, (DWORD)pfDecodeFrameEx);  // Set "SETCH_DECODE_CBEX" to channel

	return S_OK;
}
