// RotatePresetLocations.cpp : Defines the entry point for the application.
//

#include "RotatePresetLocations.h"
#include <vector>
using namespace std;

#define MAX_LOADSTRING 100

const DWORD g_kMaxElapse = 3;

CMPUT                 *g_pMPUT = NULL;
TFISHEYEVIEWER        g_tViewer;
TFISHEYEPRESENTPARAMS g_tPresentParams;
TFISHEYEROTATEOPTIONS g_tRotateOptions;
POINTS                g_MouseLBtnBeginPos;
POINTS                g_MouseLBtnCheckPos;
POINTS                g_MouseMovePos;
DWORD                 g_MouseElapseX;
DWORD                 g_MouseElapseY;
HCURSOR               g_hCursHand;
RECT                  g_WindowRoi;

// Array of fisheye control parameters
typedef std::vector<TFISHEYECONTROLPARAMS*> VFISHEYECTRLPARAMS;
typedef std::vector<TFISHEYECONTROLPARAMS*>::iterator VFISHEYECTRLPARAMSITR;
VFISHEYECTRLPARAMS g_vPresets;

// Fisheye Stream Function Declaration
// ----------------------------------------------------------------------------------------------------
void connect();
void disconnect();
// ----------------------------------------------------------------------------------------------------

// Fisheye Viewer Function Declaration
// ----------------------------------------------------------------------------------------------------
void updateViewer(TFISHEYEVIEWER *ptViewer);
void getViewerPresentParameters(TFISHEYEPRESENTPARAMS *ptPresentParams);
void updateRotateOption(TFISHEYEROTATEOPTIONS *ptRotateOptions);
// ----------------------------------------------------------------------------------------------------

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// the main window class name

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK	About(HWND, UINT, WPARAM, LPARAM);

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

 	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_ROTATEPRESETLOCATIONS, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow))
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDC_ROTATEPRESETLOCATIONS));

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0))
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return (int) msg.wParam;
}



//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage are only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ROTATEPRESETLOCATIONS));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDC_ROTATEPRESETLOCATIONS);
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_SMALL));

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HINSTANCE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   HWND hWnd;

   hInst = hInstance; // Store instance handle in our global variable

   hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, 800, 600, NULL, NULL, hInstance, NULL);

   if (!hWnd)
   {
      return FALSE;
   }

   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);

   g_tViewer.hViewerWindow = hWnd;
   g_tPresentParams.hViewerWindow = hWnd;

   g_pMPUT = new CMPUT(hWnd);
   if (!g_pMPUT)
   {
	   return FALSE;
   }

   g_hCursHand = LoadCursor(NULL, IDC_HAND);

   return TRUE;
}

//
//  FUNCTION: WndProc(HWND, UINT, WPARAM, LPARAM)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	//PAINTSTRUCT ps;
	//HDC hdc;

	switch (message)
	{
	case WM_COMMAND:
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		// Parse the menu selections:
		switch (wmId)
		{
		case IDM_ABOUT:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_ABOUTBOX), hWnd, About);
			break;
		case IDM_EXIT:
			for (VFISHEYECTRLPARAMSITR iter = g_vPresets.begin(); iter != g_vPresets.end(); ++iter)
			{
				delete (*iter);
			}
			
			g_vPresets.clear();	
			VFISHEYECTRLPARAMS().swap(g_vPresets);
			
			if (g_pMPUT)
			{
				delete g_pMPUT;
				g_pMPUT = NULL;
			}
			DestroyWindow(hWnd);
			break;

		// Fisheye Stream Operation
		// ----------------------------------------------------------------------------------------------------
		case ID_FISHEYE_STREAM_CONNECT:
			connect();
			break;
		case ID_FISHEYE_STREAM_DISCONNECT:
			disconnect();
			break;
		// ----------------------------------------------------------------------------------------------------

		// Fisheye Viewer Function Operation: Select present mode
		// ----------------------------------------------------------------------------------------------------
		case ID_FISHEYE_PM_SINGLE_SOURCE_STRETCH:
			g_tViewer.dwPresentMode = FEPRESENTMODE_SINGLE_SOURCE_STRETCH;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_PRESENTMODE;
			updateViewer(&g_tViewer);
			break;
		case ID_FISHEYE_PM_SINGLE_PARTIALCORRECTION:
			g_tViewer.dwPresentMode = FEPRESENTMODE_SINGLE_PERSPECT_VIEW;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_PRESENTMODE;
			updateViewer(&g_tViewer);
			break;
		case ID_FISHEYE_PM_QUAD_UNCALIBRATE:
			g_tViewer.dwPresentMode = FEPRESENTMODE_QUAD_WITH_SOURCE_CONSTRAINPROP;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_PRESENTMODE;
			updateViewer(&g_tViewer);
			break;
		case ID_FISHEYE_PM_QUAD_PARTIALCORRECTION:
			g_tViewer.dwPresentMode = FEPRESENTMODE_QUAD_PERSPECT_VIEW;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_PRESENTMODE;
			updateViewer(&g_tViewer);
			break;
		// ----------------------------------------------------------------------------------------------------

		// Fisheye Viewer Function Operation: Select viewer angle
		// ----------------------------------------------------------------------------------------------------
		case ID_FISHEYE_VIEWERANGLE_180:
			g_tViewer.dwViewerAngle = FEVIEWERANGLE_180;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_VIEWERANGLE;
			updateViewer(&g_tViewer);
			break;
		case ID_FISHEYE_VIEWERANGLE_360:
			g_tViewer.dwViewerAngle = FEVIEWERANGLE_360;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_VIEWERANGLE;
			updateViewer(&g_tViewer);
			break;
		case ID_FISHEYE_VIEWERANGLE_360I:
			g_tViewer.dwViewerAngle = FEVIEWERANGLE_360I;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_VIEWERANGLE;
			updateViewer(&g_tViewer);
			break;
		
		// Fisheye Viewer Function Operation: Start/Stop rotating preset locations
		// ----------------------------------------------------------------------------------------------------
		case ID_FISHEYEROTATE_START:
			{
				TFISHEYECONTROLPARAMS *pPresetPoints = new TFISHEYECONTROLPARAMS [(int)g_vPresets.size()];

				int Count = 0;				
				for (VFISHEYECTRLPARAMSITR iter = g_vPresets.begin(); iter != g_vPresets.end(); ++iter, ++Count)
				{
					memcpy(&pPresetPoints[Count], *iter, sizeof(TFISHEYECONTROLPARAMS));
				}			

				g_tRotateOptions.dwUpdateFlags = (FEUPDATEROTATE_POINTS | FEUPDATEROTATE_TARGET | FEUPDATEROTATE_SPEED);
				g_tRotateOptions.PresetPointsCount = (DWORD)g_vPresets.size();
				g_tRotateOptions.pPresetPoints = pPresetPoints;
				g_tRotateOptions.hRotateTargetWindow = hWnd;
				g_tRotateOptions.RotateSpeed = 3;
				updateRotateOption(&g_tRotateOptions);

				delete pPresetPoints;				

				break;
			}			
		case ID_FISHEYEROTATE_STOP:
			g_tRotateOptions.dwUpdateFlags = FEUPDATEROTATE_SPEED;
			g_tRotateOptions.RotateSpeed = 0;
			updateRotateOption(&g_tRotateOptions);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	case WM_KEYDOWN:
		{
			switch (wParam)
			{
			// Use SPACEBAR to select preset location
			case VK_SPACE:
				{
					TFISHEYECONTROLPARAMS *pCtrlParams = new TFISHEYECONTROLPARAMS;
					memset(pCtrlParams, 0, sizeof(TFISHEYECONTROLPARAMS));
					
					getViewerPresentParameters(&g_tPresentParams);
					memcpy(pCtrlParams, &g_tPresentParams.pCtrlParams[g_tPresentParams.FocusQuadrant], sizeof(TFISHEYECONTROLPARAMS));

					g_vPresets.push_back(pCtrlParams);

					break;
				}
			default:
				break;
			}

			break;
		}
	case WM_PAINT:
		//hdc = BeginPaint(hWnd, &ps);
		// TODO: Add any drawing code here...
		//EndPaint(hWnd, &ps);
		break;
	case WM_SIZE:
		GetClientRect(hWnd, &g_WindowRoi);
		break;

	// ----------------------------------------------------------------------------------------------------
	case WM_LBUTTONDOWN:
		// Get the coordinate of cursor.
		g_MouseLBtnBeginPos = MAKEPOINTS(lParam);

		// Reset the elapsed distance.
		g_MouseElapseX = 0;
		g_MouseElapseY = 0;
		break;
	case WM_LBUTTONUP:		
		if ((g_MouseElapseX < g_kMaxElapse) && (g_MouseElapseY < g_kMaxElapse))
		{
			// Clik-on-image control
			// ----------------------------------------------------------------------------------------------------
			// Perform pan, tilt and rotate control in the quadrant. 
			// The quadrant is specified by BeginPoint and EndPoint.
			g_tViewer.tCtrlCommands.dwCtrlFlags          = FEPTZRCONTROL_CLICK | FEPTZRCONTROL_PTR;
			g_tViewer.tCtrlCommands.bClick               = TRUE;
			g_tViewer.tCtrlCommands.PTRParams.BeginPoint = MAKEPOINTS(lParam);
			g_tViewer.tCtrlCommands.PTRParams.EndPoint   = MAKEPOINTS(lParam);

			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_CONTROLCOMMANDS;
			updateViewer(&g_tViewer);
			// ----------------------------------------------------------------------------------------------------			

			// Set focus to the quadrant which is specified by AnchorPoint.
			g_tViewer.tCtrlCommands.dwCtrlFlags = FEPTZRCONTROL_ANCHOR;			
			g_tViewer.tCtrlCommands.AnchorPoint = MAKEPOINTS(lParam);

			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_CONTROLCOMMANDS;
			updateViewer(&g_tViewer);
		}
		break;
	case WM_LBUTTONDBLCLK:
		// Get the coordinate of cursor.
		g_MouseLBtnBeginPos = MAKEPOINTS(lParam);

		// Reset the control commands in the quadrant which is specified by FocusPoint.
		g_tViewer.tCtrlCommands.dwCtrlFlags            = FEPTZRCONTROL_RESET;
		g_tViewer.tCtrlCommands.ResetParams.FocusPoint = g_MouseLBtnBeginPos;
		g_tViewer.tCtrlCommands.ResetParams.bResetP    = TRUE;
		g_tViewer.tCtrlCommands.ResetParams.bResetT    = TRUE;
		g_tViewer.tCtrlCommands.ResetParams.bResetZ    = TRUE;
		g_tViewer.tCtrlCommands.ResetParams.bResetR    = TRUE;

		g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_CONTROLCOMMANDS;
		updateViewer(&g_tViewer);
		break;
	case WM_MOUSEMOVE:
		// Get the coordinate of cursor.
		g_MouseMovePos = MAKEPOINTS(lParam);

		// Just use the hand-style cursor to enhance the visual effect when controlling...
		if ((g_MouseMovePos.x > g_WindowRoi.left) && (g_MouseMovePos.x < g_WindowRoi.right) && 
			(g_MouseMovePos.y > g_WindowRoi.top) && (g_MouseMovePos.y < g_WindowRoi.bottom))
		{
			SetCursor(g_hCursHand);
		}

		if (wParam == MK_LBUTTON)
		{
			// Get the coordinate of cursor. (The left mouse button is down)
			g_MouseLBtnCheckPos = MAKEPOINTS(lParam);

			// Drag-and-drop control
			// ----------------------------------------------------------------------------------------------------
			if ((g_MouseElapseX > g_kMaxElapse) || (g_MouseElapseY > g_kMaxElapse))
			{
				// Perform pan, tilt and rotate control in the quadrant. 
				// The quadrant is specified by BeginPoint and EndPoint.
				g_tViewer.tCtrlCommands.dwCtrlFlags          = FEPTZRCONTROL_CLICK | FEPTZRCONTROL_PTR;
				g_tViewer.tCtrlCommands.bClick               = FALSE;
				g_tViewer.tCtrlCommands.PTRParams.BeginPoint = g_MouseLBtnBeginPos;
				g_tViewer.tCtrlCommands.PTRParams.EndPoint   = g_MouseLBtnCheckPos;

				g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_CONTROLCOMMANDS;
				updateViewer(&g_tViewer);
			}			
			// ----------------------------------------------------------------------------------------------------

			// Update the elapsed distance.
			g_MouseElapseX += abs(g_MouseLBtnCheckPos.x - g_MouseLBtnBeginPos.x);
			g_MouseElapseY += abs(g_MouseLBtnCheckPos.y - g_MouseLBtnBeginPos.y);

			// Update the begin position!
			g_MouseLBtnBeginPos = g_MouseLBtnCheckPos;
		}
		break;
	case WM_MOUSEWHEEL:
		// Perform zoom control in the quadrant which is specified by FocusPoint.
		g_tViewer.tCtrlCommands.dwCtrlFlags        = FEPTZRCONTROL_Z;
		g_tViewer.tCtrlCommands.ZParams.FocusPoint = g_MouseMovePos;
		g_tViewer.tCtrlCommands.ZParams.DeltaSteps = GET_WHEEL_DELTA_WPARAM(wParam) / 16;

		g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_CONTROLCOMMANDS;
		updateViewer(&g_tViewer);
		break;
	// ----------------------------------------------------------------------------------------------------
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Message handler for about box.
INT_PTR CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(lParam);
	switch (message)
	{
	case WM_INITDIALOG:
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}

// Fisheye Stream Function Definition
// ----------------------------------------------------------------------------------------------------
void connect()
{
	if (g_pMPUT)
	{
		BOOL bSupportFisheye;
		bSupportFisheye = g_pMPUT->FisheyeCheckSupport();
		if (!bSupportFisheye)
		{
			MessageBox(g_tViewer.hViewerWindow, "Not support fisheye render!", NULL, (MB_OK | MB_ICONWARNING));
			return;
		}

		g_pMPUT->FisheyeEnableRender();

		g_pMPUT->Connect();
	}
}

void disconnect()
{
	if (g_pMPUT)
	{
		g_pMPUT->Disconnect();
	}
}
// ----------------------------------------------------------------------------------------------------

// Fisheye Viewer Function Definition
// ----------------------------------------------------------------------------------------------------
void updateViewer(TFISHEYEVIEWER *ptViewer)
{
	if (g_pMPUT)
	{
		g_pMPUT->FisheyeUpdateViewer(ptViewer);
	}
}

void getViewerPresentParameters(TFISHEYEPRESENTPARAMS *ptPresentParams)
{
	if (g_pMPUT)
	{
		g_pMPUT->FisheyeGetViewerPresentParameters(ptPresentParams);
	}
}

void updateRotateOption(TFISHEYEROTATEOPTIONS *ptRotateOptions)
{
	if (g_pMPUT)
	{
		g_pMPUT->FisheyeUpdateRotateOptions(ptRotateOptions);
	}
}
// ----------------------------------------------------------------------------------------------------
