#include <conio.h>
#include "tchar.h"
#include "DataBroker.h"
#include "PacketMaker.h"
#include "mainprofile_error.h"

#define MAX_IP_LENGTH 256
#define MAX_USERNAME_LENGTH 256
#define MAX_PASSWORD_LENGTH 256


// This callback function is used to receive media packet, the calling convention must be stdcall
SCODE __stdcall DataBrokerAVCallBack(DWORD dwContext,TMediaDataPacketInfo *ptMediaDataPacket)
{
   return S_OK;
}

// This callback function is used to receive status info, the calling convention must be stdcall
SCODE __stdcall DataBrokerStatusCallBack(DWORD dwContext, TDataBrokerStatusType tStatusType, PVOID pvParam1, PVOID pvParam2)
{
	switch (tStatusType)
	{
		case eOnConnectionInfo:
		{
			printf("Get eOnConnectionInfo, start to receive packet\n");
		}
		break;

		case eOnStopped:
		{
			printf("Get eOnStopped, stop to receive packet\n");
		}
		break;

		case eOnTxChannelStart:
		{
			printf("Talk start\n");
		}
		break;
		
		case eOnTxChannelClosed:
		{
			printf("Talk stop\n");
		}
		break;

		case eOnChangeMediaType:
		{
			printf("Media type change, Please stop talk \n");
		}
		break;

		default:
		{
		}	
		break;
	}

   return S_OK;
}

// This callback function  is used to transmite media, the calling convention must be stdcall
SCODE __stdcall DataBrokerTxCallBack(DWORD dwContext, BYTE **ppbyData, DWORD *pdwLen, DWORD *pdwDataTimePeriod)
{
	*pdwLen = 0;
	*ppbyData = 0;
	*pdwDataTimePeriod = 0;
    
	// If use audio capture to receive audio frame, need to return DATABROKER_S_NOMORE_CALLBACK
	return DATABROKER_S_NOMORE_CALLBACK;
}

// This callback function is used to receive audio frame from audio capture, the calling convention must be stdcall
SCODE __stdcall StaticAudioCapturePacketCB(DWORD dwContent, TMediaDataPacketInfo *ptMediaPacket, DWORD dwDataTimePeriod)
{
	HANDLE*	phConn = (HANDLE*) dwContent;

	if (*phConn != NULL)
	{
		// Input audio frame to connection for talk
		DataBroker_InputTxPacket(*phConn, ptMediaPacket, dwDataTimePeriod);
	}

	return S_OK;
}

// This callback function  is used to receive status info from audio capture, the calling convention must be stdcall
SCODE __stdcall StaticAudioCaptureStatusCB(DWORD dwContent, DWORD dwStatus, DWORD dwParam1, DWORD dwParam2)
{
	// do nothing here
	return S_OK;
}

void CreateConnectionOption(TDataBrokerConnectionOptions *ptOpt)
{	
	DWORD dwAudioEnc = 0;
	
	//Set device IP
	printf("Please input device IP: ");
	gets(ptOpt->pzIPAddr);

	//Set device username
	printf("Please input device user name (press [ENTER] for empty): ");
	gets(ptOpt->pzUID);

	//Set device password
	printf("Please input device password (press [ENTER] for empty): ");
	gets(ptOpt->pzPWD);

	//Set device HTTP port
	printf("Please input device HTTP port: ");
	_tscanf("%hu", &(ptOpt->wHttpPort));

	//Set audio encode type
	printf("Please input audio encode type (G711 = 1, G729A = 2): ");
	_tscanf("%u", &dwAudioEnc);

	if (dwAudioEnc == 1)
	{
		ptOpt->dwAudioEnc = mctG711;
	}
	else if (dwAudioEnc == 2)
	{
        ptOpt->dwAudioEnc = mctG729A;
	}
	else
	{
        printf("Input audio encode type error \n");
		ptOpt->dwAudioEnc = mctG711;
	}

	//Set connection protocol type to TCP, you may refer to TsdrProtocolType for other protocols.
	ptOpt->dwProtocolType = eptTCP;

	//Set connection media type to Video and Audio.
	ptOpt->dwMediaType =  (emtVideo | emtAudio);

	//Set device server type to auto, the DataBroker will auto detect the server type.
	ptOpt->pzServerType = _T("auto");

	//Pass context to DataBrokerStatusCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwStatusContext = NULL;

	//Pass context to DataBrokerAVCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwAVContext = NULL;

	//Pass context to DataBrokerTxCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwTxContext = NULL;

	//Set status callback to connection option
	ptOpt->pfStatus = DataBrokerStatusCallBack;

	//Set Audio/Video callback to connection option
	ptOpt->pfAV = DataBrokerAVCallBack;

	//Set transmitting media callback to connection option
	ptOpt->pfTx = DataBrokerTxCallBack;
	
	//Set dwFlags to tell DataBroker what options we set.
	ptOpt->dwFlags = eConOptProtocolAndMediaType | eConOptHttpPort | 
		(eConOptStatusCallback | eConOptStatusContext) |
		(eConOptAVCallback | eConOptAVContext)|
		(eConOptTxCallback | eConOptTxContext | eConOptAudioEncCodec);
}

void CreateAudioCaptureOption(TPMACAPCHANNELOPTION *ptOpt, DWORD dwAudioEnc, HANDLE* phConn)
{
	if (dwAudioEnc == mctG711)
	{
		ptOpt->dwAudioFramesPerPacket = 1;
		ptOpt->dwAudioSizePerFrame = 640;
		ptOpt->dwAudioCodec = mctG711;
	}
	else
	{
		ptOpt->dwAudioFramesPerPacket = 10;
		ptOpt->dwAudioSizePerFrame = 160;
		ptOpt->dwAudioCodec = mctG729A;
	}

	ptOpt->dwAudioChannelNum = 1;
	ptOpt->dwAudioSampleFreq = 8000;
	ptOpt->dwBitsPerSample = 16;
	ptOpt->dwPacketContext = (DWORD) phConn;
	ptOpt->dwStatusContext = NULL;
	ptOpt->pfPacketCB = StaticAudioCapturePacketCB;
	ptOpt->pfStatusCB = StaticAudioCaptureStatusCB;
}


int main()
{
	TCHAR szIP[MAX_IP_LENGTH] = {0};
	TCHAR szUserName[MAX_PASSWORD_LENGTH] = {0};
	TCHAR szPassword[MAX_USERNAME_LENGTH] = {0};

	SCODE   scRet = S_FAIL;
	HANDLE	hDataBroker = NULL;
	HANDLE	hConn = NULL;

	HANDLE hPacketMaker = NULL;
	HANDLE hAudioCapCh = NULL; 

	int nMaxChannel = 32;
	
	
	// STEP 1: Initial DataBroker.
	scRet = DataBroker_Initial(&hDataBroker, nMaxChannel, DataBrokerStatusCallBack, DataBrokerAVCallBack, mctALLCODEC, 0, DATABROKER_VERSION);
	
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Initial failed with error %X \n", scRet);
		return -1;
	}
	
	// STEP 2: Create connection.
	scRet = DataBroker_CreateConnection(hDataBroker, &hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_CreateConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 3: Set option to connection.
	TDataBrokerConnectionOptions tOpt;
	memset(&tOpt, 0, sizeof(tOpt));
	tOpt.pzIPAddr = szIP;
	tOpt.pzUID = szUserName;
	tOpt.pzPWD = szPassword;

    CreateConnectionOption(&tOpt);               
	scRet = DataBroker_SetConnectionOptions(hConn, &tOpt);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionOptions failed with error %X \n", scRet);
		return -1;
	}
    
	// STEP 4: Initial PacketMaker.
	scRet = PacketMaker_Initial(&hPacketMaker, 0, PACKET_MAKER_VERSION);

	if (IS_FAIL(scRet))
	{
		printf("PacketMaker_Initial failed with error %X \n", scRet);
		return -1;
	}

    // STEP 5: Set audio capture option and create audio capture channel.
	TPMACAPCHANNELOPTION tChOpt;
	memset(&tChOpt, 0, sizeof(tChOpt));

    CreateAudioCaptureOption(&tChOpt, tOpt.dwAudioEnc, &hConn);
	scRet = PacketMaker_CreateAudioCaptureChannel(hPacketMaker, &hAudioCapCh, &tChOpt);

	if (IS_FAIL(scRet))
	{
		printf("PacketMaker_CreateAudioCaptureChannel failed with error %X \n", scRet);
		return -1;
	}

	// STEP 6: Connect to device, this function is an asynchronous function, 
	//		   the connection information should be get in StatusCallback function we set.
	scRet = DataBroker_Connect(hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Connect failed with error %X \n", scRet);
		return -1;
	}

	printf("Press t key to start talk\n");
	printf("Press s key to stop talk\n");
	printf("Press e key to stop connect\n");

	char szbuffer[32] = {0};

	while (1)
	{
		gets(szbuffer);

		if (szbuffer[0] == 't')
		{
			// STEP 7: Start audio capture, this function is an asynchronous function, 
			// the audio frame should be get in StaticAudioCapturePacketCB.
			scRet = PacketMaker_StartAudioCapture(hAudioCapCh);

			if (IS_FAIL(scRet))
			{
				printf("PacketMaker_StartAudioCapture failed with error %X \n", scRet);
				return -1;
			}
           
			// STEP 8: Start talk to device, this function is an asynchronous function, 
			// the media be transmitted in DataBrokerTxCallBack.
			scRet = DataBroker_StartTxConnection(hConn);

			if (scRet != S_OK)
			{
				printf("DataBroker_StartTxConnection failed with error %X \n", scRet);
			}
		}
		else if (szbuffer[0] == 's')
		{
			// STEP 9: Stop audio capture
			scRet = PacketMaker_StopAudioCapture(hAudioCapCh);

			if (IS_FAIL(scRet))
			{
				printf("PacketMaker_StopAudioCapture failed with error %X \n", scRet);
				return -1;
			}

			// STEP 10: Stop talk to device
			scRet = DataBroker_StopTxConnection(hConn);

			if (scRet != S_OK)
			{
				printf("DataBroker_StopTxConnection failed with error %X \n", scRet);
			}
		}
		else if (szbuffer[0] == 'e')
		{
			break;
		}
	}

	// STEP 11: Disconnect to device
	scRet = DataBroker_Disconnect(hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Disconnect failed with error %X \n", scRet);
		return -1;
	}

	// STEP 12: Delete audio capture channel
	scRet = PacketMaker_DeleteAudioCaptureChannel(&hAudioCapCh);

	if (IS_FAIL(scRet))
	{
		printf("PacketMaker_DeleteAudioCaptureChannel failed with error %X \n", scRet);
		return -1;
	}

	// STEP 13: Release PacketMaker
	scRet = PacketMaker_Release(&hPacketMaker);

	if (IS_FAIL(scRet))
	{
		printf("PacketMaker_Release failed with error %X \n", scRet);
		return -1;
	}

    // STEP 14: Delete connection
    scRet = DataBroker_DeleteConnection(hDataBroker, &hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_DeleteConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 15: Release DataBroker
	scRet = DataBroker_Release(&hDataBroker);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Release failed with error %X \n", scRet);
		return -1;
	}

	return 0;
}