#include "OSDefine.h"
#include <string.h>
#include "DataBroker.h"
#include "PacketMaker.h"
#include "mainprofile_error.h"

#define MAX_IP_LENGTH 256
#define MAX_USERNAME_LENGTH 256
#define MAX_PASSWORD_LENGTH 256
#define MAX_FILEPATH_LENGTH 256
#define AUDIO_OUTPUT_BUFFER_SIZE 4096

typedef	struct TMakePacketInfo
{
	// Pointer to current read position of the WAV buffer
	BYTE* pbyCurrPos;

	// Pointer to the end position of the WAV buffer
	BYTE* pbyEndPos;

	// Pointer to the temporary buffer for PacketMaker to generate packet
	BYTE* pbyOutputBitstream;

	// Size of the temporary buffer
	DWORD dwOutputBitstreamSize;	

	// Audio Channel handle of PacketMaker
	HANDLE hAudioCh;

	// End of file flag
	bool bEndFile;

	TMakePacketInfo():
	pbyCurrPos(NULL),
	pbyEndPos(NULL),
    pbyOutputBitstream(NULL),
	dwOutputBitstreamSize(0),
	hAudioCh(NULL),
	bEndFile(false)
	{

	}

} TMakePacketInfo;

// This callback function is used to receive media packet, the calling convention must be stdcall
SCODE __stdcall DataBrokerAVCallBack(DWORD dwContext,TMediaDataPacketInfo *ptMediaDataPacket)
{
   return S_OK;
}

// This callback function is used to receive status info, the calling convention must be stdcall
SCODE __stdcall DataBrokerStatusCallBack(DWORD dwContext, TDataBrokerStatusType tStatusType, PVOID pvParam1, PVOID pvParam2)
{
	switch (tStatusType)
	{
		case eOnConnectionInfo:
		{
			printf("Get eOnConnectionInfo, start to receive packet\n");
		}
		break;

		case eOnStopped:
		{
			printf("Get eOnStopped, stop to receive packet\n");
		}
		break;

		case eOnTxChannelStart:
		{
			printf("Talk start\n");
		}
		break;
		
		case eOnTxChannelClosed:
		{
			printf("Talk stop\n");
		}
		break;

		default:
		{
		}	
		break;
	}

   return S_OK;
}


// This callback function  is used to transmit media, the calling convention must be stdcall
SCODE __stdcall DataBrokerTxCallBack(DWORD dwContext, BYTE **ppbyData, DWORD *pdwLen, DWORD *pdwDataTimePeriod)
{
	TMakePacketInfo* ptInfo = (TMakePacketInfo*) dwContext;

	DWORD dwAudioSampleFreq = 8000;
	DWORD dwBytePerSample = 2;
	SCODE scRet = S_FAIL;

	if (ptInfo == NULL)
	{
	   return S_FAIL;
	}

	if (ptInfo->pbyCurrPos < ptInfo->pbyEndPos)
	{
		TMediaDataPacketInfo	tMediaPacket;
		DWORD					dwUsedSize = 0;

		memset(&tMediaPacket, 0, sizeof(tMediaPacket));
		tMediaPacket.pbyBuff = ptInfo->pbyOutputBitstream;		
	
		scRet = PacketMaker_MakeAudioPacket(ptInfo->hAudioCh, ptInfo->pbyCurrPos, ptInfo->pbyEndPos - ptInfo->pbyCurrPos, ptInfo->dwOutputBitstreamSize, &dwUsedSize, &tMediaPacket);
		ptInfo->pbyCurrPos += dwUsedSize;

		*pdwLen = tMediaPacket.dwBitstreamSize + tMediaPacket.dwOffset;
		*ppbyData = tMediaPacket.pbyBuff;
      

		// The millisecond of data time period
		*pdwDataTimePeriod = ((dwUsedSize * 1000 )/(dwBytePerSample * dwAudioSampleFreq));

		if (ptInfo->pbyCurrPos == ptInfo->pbyEndPos)
		{
			ptInfo->bEndFile = true;
		}
	}

	return S_OK;
}


void CreateConnectionOption(TDataBrokerConnectionOptions *ptOpt, TMakePacketInfo* ptInfo)
{	
	DWORD dwAudioEnc = 0;
	
	//Set device IP
	printf("Please input device IP: ");
	gets(ptOpt->pzIPAddr);

	//Set device username
	printf("Please input device user name (press [ENTER] for empty): ");
	gets(ptOpt->pzUID);

	//Set device password
	printf("Please input device password (press [ENTER] for empty): ");
	gets(ptOpt->pzPWD);

	//Set device HTTP port
	printf("Please input device HTTP port: ");
	_tscanf("%hu", &(ptOpt->wHttpPort));

	//Set audio encode type
	printf("Please input audio encode type (G711 = 1, G729A = 2): ");
	_tscanf("%u", &dwAudioEnc);

	if (dwAudioEnc == 1)
	{
		ptOpt->dwAudioEnc = mctG711;
	}
	else if (dwAudioEnc == 2)
	{
        ptOpt->dwAudioEnc = mctG729A;
	}
	else
	{
        printf("Input audio encode type error \n");
		ptOpt->dwAudioEnc = mctG711;
	}

	//Set connection protocol type to TCP, you may refer to TsdrProtocolType for other protocols.
	ptOpt->dwProtocolType = eptTCP;

	//Set connection media type to Video and Audio.
	ptOpt->dwMediaType =  (emtVideo | emtAudio);

	//Set device server type to auto, the DataBroker will auto detect the server type.
	ptOpt->pzServerType = _T("auto");

	//Pass context to DataBrokerStatusCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwStatusContext = NULL;

	//Pass context to DataBrokerAVCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwAVContext = NULL;

	//Pass context to DataBrokerTxCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwTxContext = (DWORD) ptInfo;

	//Set status callback to connection option
	ptOpt->pfStatus = DataBrokerStatusCallBack;

	//Set Audio/Video callback to connection option
	ptOpt->pfAV = DataBrokerAVCallBack;

	//Set transmitting media callback to connection option
	ptOpt->pfTx = DataBrokerTxCallBack;
	
	//Set dwFlags to tell DataBroker what options we set.
	ptOpt->dwFlags = eConOptProtocolAndMediaType | eConOptHttpPort | 
		(eConOptStatusCallback | eConOptStatusContext) |
		(eConOptAVCallback | eConOptAVContext)|
		(eConOptTxCallback | eConOptTxContext | eConOptAudioEncCodec);
}

SCODE ReadData(FILE *pfInput, BYTE **ppbyBuf, DWORD *pdwSize, DWORD dwAudioEnc)
{
	DWORD	dwFileSize = 0;
    DWORD	dwBufferSize = 0;
	BYTE	*pbyBuf = NULL;
	const DWORD	dwWAVHeaderSize = 44;

	fseek(pfInput, 0, SEEK_END);
	dwFileSize = ftell(pfInput) - dwWAVHeaderSize;
	fseek(pfInput, dwWAVHeaderSize, SEEK_SET);

	if (mctG729A == dwAudioEnc)
	{
       dwBufferSize = dwFileSize + (1600 - dwFileSize % 1600);
	}
	else
	{
       dwBufferSize = dwFileSize + (1280 - dwFileSize % 1280);
	}

	pbyBuf = (BYTE *) malloc(dwBufferSize);

	if (!pbyBuf)
	{
		printf("malloc fail \n");
		return S_FAIL;
	}

	memset(pbyBuf, 0, dwBufferSize);

	if (fread(pbyBuf, dwFileSize, 1, pfInput) < 1)
	{
		printf("Read file error\n");
		free(pbyBuf);
		return S_FAIL;
	}

	*ppbyBuf = pbyBuf;
	*pdwSize = dwBufferSize;

	return S_OK;
}

SCODE FreeData(BYTE **ppbyBuf)
{
	if (*ppbyBuf == NULL)
	{
		return S_FAIL;
	}

	free(*ppbyBuf);
	*ppbyBuf = NULL;

	return S_OK;
}

void CreateAudioChannelOption(TPMCHANNELOPTION *ptOpt, DWORD dwAudioEnc)
{
	if (dwAudioEnc == mctG711)
	{
		ptOpt->dwAudioFramesPerPacket = 1;
		ptOpt->dwAudioSizePerFrame = 640;
		ptOpt->dwAudioCodec = mctG711;
	}
	else
	{
		ptOpt->dwAudioFramesPerPacket = 10;
		ptOpt->dwAudioSizePerFrame = 160;
		ptOpt->dwAudioCodec = mctG729A;
	}

	ptOpt->dwAudioSampleFreq = 8000;
	ptOpt->dwAudioChannelNum = 1;		
	ptOpt->dwFlag = eChOptAudioCodec| eChOptAudioSampleFreq | eChOptAudioChannelNum | eChOptAudioFramePerPacket | eChOptAudioSizePerFrame;
}


int main()
{
	TCHAR szIP[MAX_IP_LENGTH] = {0};
	TCHAR szUserName[MAX_PASSWORD_LENGTH] = {0};
	TCHAR szPassword[MAX_USERNAME_LENGTH] = {0};
	TCHAR szFilePath[MAX_FILEPATH_LENGTH] = {0};

	SCODE   scRet = S_FAIL;
	HANDLE	hDataBroker = NULL;
	HANDLE	hConn = NULL;

	HANDLE hPacketMaker = NULL;
	
	FILE *pfInput = NULL;
	BYTE *pbyInBuffer = NULL;
	DWORD dwInputSize = 0;

	BYTE abyOutBuff[AUDIO_OUTPUT_BUFFER_SIZE] = {0};

	TMakePacketInfo tMakePacketInfo;
	int nMaxChannel = 32;

	//Get audio file path
	printf("Input audio format:16 bits, signed, little endian, 8000 Hz Mono\n");
	printf("Please input audio file: ");
	gets(szFilePath);

	
	// STEP 1: Initial DataBroker.
	scRet = DataBroker_Initial(&hDataBroker, nMaxChannel, DataBrokerStatusCallBack, DataBrokerAVCallBack, mctALLCODEC, 0, DATABROKER_VERSION);
	
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Initial failed with error %X \n", scRet);
		return -1;
	}
	
	// STEP 2: Create connection.
	scRet = DataBroker_CreateConnection(hDataBroker, &hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_CreateConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 3: Set option to connection.
	TDataBrokerConnectionOptions tOpt;
	memset(&tOpt, 0, sizeof(tOpt));
	tOpt.pzIPAddr = szIP;
	tOpt.pzUID = szUserName;
	tOpt.pzPWD = szPassword;

    CreateConnectionOption(&tOpt, &tMakePacketInfo);               
	scRet = DataBroker_SetConnectionOptions(hConn, &tOpt);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionOptions failed with error %X \n", scRet);
		return -1;
	}

	// STEP 4: Initial PacketMaker.
	scRet = PacketMaker_Initial(&hPacketMaker, 0, PACKET_MAKER_VERSION);

	if (IS_FAIL(scRet))
	{
		printf("PacketMaker_Initial failed with error %X \n", scRet);
		return -1;
	}

	// STEP 5: Set channel option and create audio channel.
	TPMCHANNELOPTION tChOpt;
	memset(&tChOpt, 0, sizeof(tChOpt));

	CreateAudioChannelOption(&tChOpt, tOpt.dwAudioEnc);
	scRet = PacketMaker_CreateChannel(hPacketMaker, &tMakePacketInfo.hAudioCh, &tChOpt);

	if (IS_FAIL(scRet))
	{
		printf("PacketMaker_CreateChannel failed with error %X \n", scRet);
		return -1;
	}
    
	// STEP 6: Open file and read data to buffer.
	pfInput = fopen(szFilePath, "rb");

	if (pfInput == NULL)
	{
		printf("Open file failed \n");
		return -1;
	}

	scRet = ReadData(pfInput, &pbyInBuffer, &dwInputSize, tOpt.dwAudioEnc);
	
	if (scRet != S_OK)
	{
		printf("Read data failed from file \n");
		return -1;
	}

	// STEP 7: Connect to device, this function is an asynchronous function, 
	//		   the connection information should be get in StatusCallback function we set.
	scRet = DataBroker_Connect(hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Connect failed with error %X \n", scRet);
		return -1;
	}

	printf("Press t key to start talk\n");
	printf("Press e key to stop connect\n");

	char szbuffer[32] = {0};

	while (1)
	{
		gets(szbuffer);

		if (szbuffer[0] == 't')
		{
			// STEP 8: Set buffer and buffer size to MakePacketInfo
			tMakePacketInfo.pbyCurrPos = pbyInBuffer;
			tMakePacketInfo.pbyEndPos = pbyInBuffer + dwInputSize;
			tMakePacketInfo.pbyOutputBitstream = abyOutBuff;
			tMakePacketInfo.dwOutputBitstreamSize = AUDIO_OUTPUT_BUFFER_SIZE;
			tMakePacketInfo.bEndFile = false;
		
			// STEP 9: Start talk to device, this function is an asynchronous function, 
			// the media be transmitted in DataBrokerTxCallBack.
			scRet = DataBroker_StartTxConnection(hConn);

			if (scRet != S_OK)
			{
				printf("DataBroker_StartTxConnection failed with error %X \n", scRet);
			}

			//To check if end of file
			
			while (1)
			{
				if (tMakePacketInfo.bEndFile)
				{
					// STEP 10: Stop talk to device
					scRet = DataBroker_StopTxConnection(hConn);

					if (scRet != S_OK)
					{
						printf("DataBroker_StopTxConnection failed with error %X \n", scRet);
					}

					break;
				}
				else
				{
					Sleep(16);
				}
			}
		}
		else if (szbuffer[0] == 'e')
		{
			break;
		}
	}

	// STEP 11: Disconnect to device
	scRet = DataBroker_Disconnect(hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Disconnect failed with error %X \n", scRet);
		return -1;
	}

	// STEP 12: Delete audio channel
	scRet = PacketMaker_DeleteChannel(&tMakePacketInfo.hAudioCh);

	if (IS_FAIL(scRet))
	{
		printf("PacketMaker_DeleteChannel failed with error %X \n", scRet);
		return -1;
	}

	// STEP 13: Release PacketMaker
	scRet = PacketMaker_Release(&hPacketMaker);

	if (IS_FAIL(scRet))
	{
		printf("PacketMaker_Release failed with error %X \n", scRet);
		return -1;
	}

	// STEP 14: Free buffer
	scRet = FreeData(&pbyInBuffer);

	if (IS_FAIL(scRet))
	{
		printf("PFreeData failed with error %X \n", scRet);
		return -1;
	}

    // STEP 15: Delete connection
    scRet = DataBroker_DeleteConnection(hDataBroker, &hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_DeleteConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 16: Release DataBroker
	scRet = DataBroker_Release(&hDataBroker);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Release failed with error %X \n", scRet);
		return -1;
	}

	return 0;
}
