#include <cstdio>
#include <cassert>

#include "parsedatapacket.h"

#ifndef HTONL
#define HTONL(x) \
	(((DWORD)x) << 24) + \
	((((DWORD)x) & 0xff00) << 8) + \
	((((DWORD)x) >> 8) & 0xff00) + \
	(((DWORD)x) >> 24)
#endif

#ifndef NTOHL
#define NTOHL(x) \
	(((DWORD)x) << 24) + \
	((((DWORD)x) & 0xff00) << 8) + \
	((((DWORD)x) >> 8) & 0xff00) + \
	(((DWORD)x) >> 24)
#endif

void Sample_SaveV3Packet(const char* pszFilename, const TMediaDataPacketInfoV3* ptPacket)
{
	FILE* fp = fopen(pszFilename, "wb");

	assert(fp);

	DWORD dwDummyData = 0; // NOTE: for buffer is null

	// STEP 1: write out pbyTLVExt
	if (ptPacket->tIfEx.tRv1.tExt.pbyTLVExt != NULL)
	{
		fwrite(ptPacket->tIfEx.tRv1.tExt.pbyTLVExt, 1, ptPacket->tIfEx.tRv1.tExt.dwTLVExtLen, fp);
	}
	else
	{
		fwrite(&dwDummyData, 1, sizeof(dwDummyData), fp);
	}

	// STEP 2: write out pbyBuff
	if (ptPacket->tIfEx.tInfo.pbyBuff != NULL)
	{
		fwrite(ptPacket->tIfEx.tInfo.pbyBuff, 1, ptPacket->tIfEx.tInfo.dwOffset + ptPacket->tIfEx.tInfo.dwBitstreamSize, fp);
	}
	else
	{
		fwrite(&dwDummyData, 1, sizeof(dwDummyData), fp);
	}

	// STEP 3: write out pbyVExtBuf
	if (ptPacket->pbyVExtBuf != NULL)
	{
		DWORD dwBEVExtLen = HTONL(ptPacket->dwVExtLen);
		fwrite(&dwBEVExtLen, 1, sizeof(DWORD), fp);
		fwrite(ptPacket->pbyVExtBuf, 1, ptPacket->dwVExtLen, fp);
	}
	else
	{
		fwrite(&dwDummyData, 1, sizeof(dwDummyData), fp);
	}

	fclose(fp);
}

void Sample_LoadV3Packet(const char* pszFilename, TMediaDataPacketInfoV3* ptPacket, BYTE* pbyBuff, BYTE* pbyTLVExt, BYTE* pbyVExtBuf)
{
	FILE* fp = fopen(pszFilename, "rb");

	assert(fp);

	// STEP 1: initial packet
	memset(ptPacket, 0, sizeof(TMediaDataPacketInfoV3));

	ptPacket->tIfEx.tRv1.tExt.dwStructSize = sizeof(TMediaDataPacketInfoV3);

	// STEP 2: read into pbyTLVExt
	fread(&ptPacket->tIfEx.tRv1.tExt.dwTLVExtLen, 1, sizeof(DWORD), fp);

	if (ptPacket->tIfEx.tRv1.tExt.dwTLVExtLen != 0)
	{
		memcpy(pbyTLVExt, &ptPacket->tIfEx.tRv1.tExt.dwTLVExtLen, sizeof(DWORD));

		ptPacket->tIfEx.tRv1.tExt.dwTLVExtLen = NTOHL(ptPacket->tIfEx.tRv1.tExt.dwTLVExtLen);

		fread(pbyTLVExt + sizeof(DWORD), 1, ptPacket->tIfEx.tRv1.tExt.dwTLVExtLen, fp);

		ptPacket->tIfEx.tRv1.tExt.pbyTLVExt = pbyTLVExt;
	}

	ptPacket->tIfEx.tRv1.tExt.dwTLVExtLen += sizeof(DWORD);

	// STEP 3: read into pbyBuff
	DWORD dwFrameSize = 0;

	fread(&dwFrameSize, 1, sizeof(DWORD), fp);

	if (dwFrameSize != 0)
	{
		memcpy(pbyBuff, &dwFrameSize, sizeof(DWORD));

		dwFrameSize = NTOHL(dwFrameSize);

		fread(pbyBuff + sizeof(DWORD), 1, dwFrameSize, fp);

		ptPacket->tIfEx.tInfo.pbyBuff = pbyBuff;
	}

	// STEP 4: read into pbyVExtBuf
	DWORD dwVExtLen = 0;

	fread(&dwVExtLen, 1, sizeof(DWORD), fp);

	if (dwVExtLen)
	{
		dwVExtLen = NTOHL(dwVExtLen);

		fread(pbyVExtBuf, 1, dwVExtLen, fp);

		ptPacket->pbyVExtBuf = pbyVExtBuf;
	}

	fclose(fp);

	// STEP 5: reconstruct packet
	DataPacket_ParseV3(ptPacket);
}

int main(int argc, char* argv[])
{
	const char szFilename[] = "raw_media_packet.bin";

	char szFilepath[FILENAME_MAX] = {0};

	size_t nPathLen = strrchr(argv[0], '\\') - argv[0] + 1;

	strncpy(szFilepath, argv[0], nPathLen);

	strncpy(szFilepath + nPathLen, szFilename, sizeof(szFilename));

	// allocate enough buffer for packet
	BYTE* pbyBuff = (BYTE *) malloc(16384);
	BYTE* pbyTLVExt = (BYTE *) malloc(4096);
	BYTE* pbyVExtBuf = (BYTE *) malloc(4096);

	TMediaDataPacketInfoV3 tPacket;

	Sample_LoadV3Packet(szFilepath, &tPacket, pbyBuff, pbyTLVExt, pbyVExtBuf);

	Sample_SaveV3Packet(szFilepath, &tPacket);

	free(pbyBuff);
	free(pbyTLVExt);
	free(pbyVExtBuf);

	return 0;
}
